# Cortex General Settings

The Cortex General Settings Module introduces a plugin system to create general site settings forms to save common site settings.

## Create General Settings plugins

When creating a new general settings plugin. The settings form for this plugin
will automatically being added with a tab in the General Settings form (/admin/config/general-settings)

1. Create a new module.
2. Add a dependency on the Cortex General Settings module in {custom_module}.info and in your module's composer.json
3. Create your Plugin class in src/Plugin/CortexGeneralSettings
4. Your class needs to extend \Drupal\cortex_general_settings\CortexGeneralSettingsPluginBase
5. An annotation portion to define the plugin id, label and description.
```php
/**
 *
 * @CortexGeneralSettings(
 *   id = "your_module_settings",
 *   label = @Translation("Your Module Settings"),
 *   description = @Translation("Your Module Settings description")
 * )
 */
```
6. A public function settingsForm to append form fields to the general settings form.
```php
public function settingsForm(FormStateInterface $form_state, &$complete_form): array {
  ...
}
```
7. The form fields by default are being stored as a key value array in the database
via Drupal State API. To get the stored values for your Plugin:
  * In the plugin class itself:
    * you can call `$this->getSettings()` to get all plugin values.
    * you can call `$this->getSetting('keyname')` to get a specific value by its key
  * In other places through `\Drupal::config("cortex_general_settings.your_plugin_id");`
    * Although settings added via the plugin's settings form are saved in the database through Drupal State Api,
   Getting the values can be done via the config system because the module implements the config override system.
   This allows to set default values in config files.
  * In Twig templates, the values will be available as a twig array variables `cortex_general_settings.your_plugin_id`

## Full example of a CortexGeneralSettings Plugin

```php
<?php

declare(strict_types=1);

namespace Drupal\your_module\Plugin\CortexGeneralSettings;

use Drupal\Core\Form\FormStateInterface;
use Drupal\cortex_general_settings\CortexGeneralSettingsPluginBase;

/**
 * Plugin implementation of the cortex_general_settings.
 *
 * @CortexGeneralSettings(
 *   id = "your_module_settings",
 *   label = @Translation("Your Module Settings"),
 *   description = @Translation("Your Module Settings description")
 * )
 */
final class YourSettingsPlugin extends CortexGeneralSettingsPluginBase {

  /**
   * @inheritDoc
   */
  public function settingsForm(FormStateInterface $form_state, &$complete_form): array {
    $pluginId = $this->getPluginId();
    $settingsForm = parent::settingsForm($form_state, $complete_form);

    // Set the weight for defining the position of your plugin's tab in the general settings form
    $settingsForm[$pluginId]['#weight'] = -100;

    $settingsForm[$pluginId]['your_setting'] = [
      '#type' => 'textfield',
      '#title' => $this->t('A setting you want to save'),
      '#default_value' => $this->getSetting('your_settings'),
    ];

    return $settingsForm;
  }

}
```
## Roadmap
1. Create a drush command to generate CortexGeneralSettings Plugins.
2. API documentation.
3. Feature requests ...
