<?php

declare(strict_types=1);

namespace Drupal\cortex_general_settings;

use Drupal\Component\Plugin\PluginBase;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Base class for cortex_general_settings plugins.
 */
abstract class CortexGeneralSettingsPluginBase extends PluginBase implements ContainerFactoryPluginInterface, CortexGeneralSettingsInterface {
  use StringTranslationTrait;

  /**
   * The plugin settings.
   *
   * @var ImmutableConfig
   */
  protected $settings;

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, ConfigFactoryInterface $config_factory) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->settings = $config_factory->getEditable($plugin_definition['configName']);
  }

  /**
   * {@inheritDoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('config.factory')
    );
  }

  /**
   * Get the plugin configuration.
   */
  private function config(?array $values = NULL): array {

    if ($values) {
      $settings = $this->settings->get();
      foreach ($values as $key => $value) {
        $settings[$key] = $value;
      }
      $this->settings->setData($settings);
      $this->settings->save();
    }
    if ($this->settings::class === 'Drupal\Core\Config\Config') {
      return $this->settings->get();
    }
    return $this->settings;
  }

  /**
   * {@inheritdoc}
   */
  public function label(): string {
    // Cast the label to a string since it is a TranslatableMarkup object.
    return (string) $this->pluginDefinition['label'];
  }

  /**
   * Get a Cortex setting.
   */
  public function getSetting(string $key): array | string {
    if (isset($this->config()[$key])) {
      return $this->config()[$key];
    }
    return '';
  }

  /**
   * Get all Cortex settings.
   */
  public function getSettings(): array {
    return $this->config();
  }

  /**
   * Set multiple Cortex settings.
   */
  public function saveSettings(array $settings): array {
    // Merge with existing settings.
    return $this->config($settings);
  }

  /**
   * {@inheritDoc}
   */
  public function settingsForm(FormStateInterface $form_state, &$complete_form): array {
    $settingsForm = [];
    $pluginId = $this->getPluginId();
    $definition = $this->getPluginDefinition();

    if ($this->hasAccess() && !empty($definition['form'])) {
      $subForm = \Drupal::formBuilder()->getForm($definition['form'], $this->getSettings(), $pluginId);
      $settingsForm[$pluginId]['#weight'] = -100;
      $settingsForm[$pluginId]['form']['#markup'] = \Drupal::service('renderer')->render($subForm);
    }
    return $settingsForm;
  }

  /**
   * {@inheritDoc}
   */
  public function hasAccess() : bool {
    $pluginId = $this->getPluginId();
    return \Drupal::currentUser()->hasPermission("administer cortex $pluginId settings") || \Drupal::currentUser()->hasPermission('administer cortex general settings');
  }

  /**
   * {@inheritDoc}
   */
  public function hasTranslationAccess() : bool {
    $pluginId = $this->getPluginId();
    return \Drupal::currentUser()->hasPermission("translate cortex $pluginId settings");
  }

  /**
   * {@inheritDoc}
   */
  public function submitSettingsForm(array $formValues): void {
    $this->saveSettings($formValues);
  }

}
